/*********************************************************************
 Copyright (C) 2003 Imperial College London

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
***********************************************************************/

package vishnu.server.Search;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Hits;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;


import java.util.Vector;
import java.util.Hashtable;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;


public class LUCSearcher implements SearchInterface
{
    protected Document doc=null;
    private String collection_home;
    private String index_dir;
    private Vector docIDs;
    private Vector docDescs;
    private Hits hits;
    private Hashtable ht;
    
    public LUCSearcher(String coll_home, String view)
    {
	collection_home = coll_home;
	if (!collection_home.endsWith(File.separator)) {
	    collection_home += File.separator;
	}
	index_dir = collection_home + File.separator + "index" + 
	    File.separator + "luc_index" + File.separator;
	if (view != null) {
	    index_dir += view + File.separator;
	}
    }


    public Vector getDocIdentifiers()
    {
	return docIDs;
    }
    
    
    public Vector getDocDescriptions()
    {
	return docDescs;
    }	
    
    
    public Vector getDocContent(int doc_num)
    {
	System.err.println("in doc content, doc num="+doc_num);
	Integer in = null;
	if (ht==null) {
	    System.err.println("ht = null");
	}
	try {
	    in = (Integer)ht.get(new Integer(doc_num));
	} catch (Exception e) {
	    System.err.println("exception happened: "+e);
	}
	System.err.println("*");
	int id = 1;
	System.err.println("*");
	if( in != null ) id = in.intValue();
	System.err.println("hit id ="+id);
	Vector v = new Vector();

	try {
	    Directory directory = FSDirectory.getDirectory(new File(index_dir), false);

	    IndexReader reader = IndexReader.open(directory);
	    
	    Document doc = reader.document(doc_num);
	    String path = doc.get("path");
	    // make sure the slashes go the right way
	    path = path.replaceAll("[\\\\\\/]", File.separator);
	    String full_path = collection_home + path;
	    File f = new File(full_path);
	    BufferedReader dis = new BufferedReader(new InputStreamReader(new FileInputStream(f)));
	    String line;
	    
	    while( (line=dis.readLine()) != null ){
		v.addElement(line);
	    }
	    dis.close();
	} catch(IOException ex){v.addElement("Content of this document currently unavailable");}    
	return v;
    }

    
    public void search(String q)
    {
	docIDs = new Vector();
	docDescs = new Vector();
	ht = new Hashtable();
	
	try {
	    Searcher searcher = new IndexSearcher(index_dir);
	    Analyzer analyzer = new StandardAnalyzer();
	    
	    Query query = QueryParser.parse(q, "contents", analyzer);
	    System.out.println("Searching for: " + query.toString("contents") + " in " + index_dir);
	    
	    hits = searcher.search(query);
	    System.out.println(hits.length() + " total matching documents");
	    int max_docs = Math.min(MAX_DOCS,hits.length());
	    for (int i = 0; i < max_docs; i++) {

		int id = hits.id(i);
		ht.put(new Integer(id),new Integer(i));
		docIDs.addElement(new Integer(id));
		
		Document doc = hits.doc(i);
		String path = doc.get("path");
		// make sure the slashes go the right way
		path = path.replaceAll("[\\\\\\/]", File.separator);
		String full_path = collection_home + path;
		File f = new File(full_path);
		
		String line = "";
		
		try{
		    BufferedReader dis = new BufferedReader(new InputStreamReader(new FileInputStream(f)));
		    while( line.length() < MAX_DESCRIPTION_LENGTH ){
			String tmp = dis.readLine();
			line += tmp;
		    }
		    docDescs.addElement(line.substring(0,MAX_DESCRIPTION_LENGTH));
		    dis.close();
		}catch(IOException ex){docDescs.addElement(new String("No description available"));}    
	    }
	    searcher.close();
	    
	} catch (Exception ex) {ex.printStackTrace();}
    }
}







