/**
* name      TreemapPanel.java 
* author    Peter Au
* modified  Daniel Heesch (29.04.02)
*/

package vishnu.testvis.treemap.visual;

import vishnu.testvis.object.DataManager;
import vishnu.testvis.treemap.rectangle.ClusterRect;
import vishnu.testvis.visual.*;
import vishnu.testvis.object.*;
import java.util.Stack;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

public class TreemapPanel extends Canvas
{

    private int _width, _height;
    private JButton _popUpButton;
    private NavigatorPanel _navigator;
    private String _newTerms;
    Vishnu _displayFrame;
    Image _offScreenImage=null;
    boolean _init_done=false, _drillDownFlag=false, _newSearchFlag=false, _popUpFlag=false;
    Stack _visualObjs;
    
    JPopupMenu[] popups;
    PopupMenu popup;

    public TreemapPanel(int w, int h, JButton bb, NavigatorPanel n, Vishnu displayFrame)
    {
        _displayFrame = displayFrame;
        _width=w; _height=h;
        _popUpButton = bb;
        _navigator = n;
        _displayFrame = displayFrame;
        setSize(_width,_height);
        _visualObjs = new Stack();
        addMouseListener(new MouseHandler());
        _popUpButton.setActionCommand("popUp");
        _popUpButton.setEnabled(false);
        _popUpButton.setToolTipText("Go back to the previous level");
        _popUpButton.addActionListener(new ButtonHandler());
                   
        popup = new PopupMenu();
        
        add(popup);

    }

    class MouseHandler extends MouseAdapter{}

    class ButtonHandler implements ActionListener
    {
        public void actionPerformed(ActionEvent event)
        {
            String action = event.getActionCommand();
            String st;
            if (action.equals("popUp")){
                processPopUp();
            }
        }
    }
    
    public void update(Graphics oldg)
    {
        Graphics2D g = (Graphics2D) oldg;
        Visualiser currentVObj;
        
        if (_newSearchFlag)
        {
            _visualObjs.push(new Visualiser(_displayFrame.dataManager, 0F,0F, _width, _height));
            currentVObj = ((Visualiser) _visualObjs.peek());
            currentVObj.createTreemapView(createImage(_width, _height));
            
            //-- draw the treemap view on the offscreen-buffer
            currentVObj.drawTreemapView(_offScreenImage, this);
            
            _newSearchFlag = false;
        }
        else if (_drillDownFlag)
        {
            _displayFrame.refine = true;
            currentVObj = (Visualiser) _visualObjs.peek();
            
            //-- create a list of image buffer for the animated pictures
            Image iBuffer[]=new Image[Visualiser.__FRAME];
            
            for(int i=0;i<iBuffer.length;i++)
                iBuffer[i]=createImage(_width,_height);

            //-- prepare the zooming pictures of the current visaul object
            currentVObj.prepare_zooming(iBuffer, this);

            int s = _visualObjs.size();
            float x = Visualiser.__borderwidth*s,
                y = Visualiser.__borderheight*s;
         
            DataManager dm = new DataManager(_displayFrame);
            dm.query(_newTerms);
            _visualObjs.push(new Visualiser(dm,x, y, _width-x*2, _height-y*2));

                /**
                * create a treemap view for the new data
                * before doing so, we have to copy the last zooming picture to the treemap
                * this is because the zooming picture contains the borders
                */
                
            Image tmpImg = createImage(_width, _height);
            Graphics2D tmpGph = (Graphics2D) tmpImg.getGraphics();
            tmpGph.drawImage(currentVObj.getLastZoomingImage(),0,0,this);
            
            //-- now create the treemap view
            ((Visualiser) _visualObjs.peek()).createTreemapView(tmpImg);

            //-- draw the treemap view (top element of the stack) on the offscreen buffer
            ((Visualiser) _visualObjs.peek()).drawTreemapView(_offScreenImage, this);

            //-- when everyting is done, we then show the animated pictures
            //-- we use the 2nd element of the stack, not the top one
            currentVObj.show_zoomIn(g,this);

            _drillDownFlag = false;
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        }
        else if (_popUpFlag)
        {

            currentVObj = (Visualiser) _visualObjs.peek();
            //-- redraw the treemap view on the offscreen buffer, as the previous
            //-- level is gone
            currentVObj.drawTreemapView(_offScreenImage, this);
            //-- show the animated pictures
            currentVObj.show_zoomOut(g,this);
            _popUpFlag = false;
        }
        paint(g);
    }


    public void clearDisplay()
    {
        _navigator.clear();
    }


    public void processPopUp()
    {
        if (_visualObjs.size() > 1)
        {
            _displayFrame.log.log("UNREFINE_QUERY");
            //-- get rid of the top obj in the stack
            _visualObjs.pop();
            //-- reset the necessary instance variable back to the default values
            ((Visualiser) _visualObjs.peek()).setDefault();
            //-- clear the table and display
            clearDisplay();
            if (_visualObjs.size()==1) _popUpButton.setEnabled(false);
            _popUpFlag = true;
            repaint();
        }
    }

    public void newSearch()
    {
        clearDisplay();
        _popUpButton.setEnabled(false);
        _visualObjs = new Stack();
        _newSearchFlag = true;
        repaint();
    }


    private void processDrillDown(int x, int y){
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        ClusterRect r = ((Visualiser) _visualObjs.peek()).drillDown(x,y);
        //-- chose which cluster to drill down
        if (r!=null){
            //-- do sth on 'r'??? send it back to the server and do the clustering again
            _popUpButton.setEnabled(true);
            clearDisplay();
            _drillDownFlag = true;
            _newTerms = r.getClusterObj().getLabel();
            _navigator.clear();  //-- clear the table and the display
            repaint();

        }
        else
            setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }



    private void processInspect(int x, int y){
        setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        ClusterRect r = ((Visualiser) _visualObjs.peek()).inspect(x,y);
        if (r!=null){
            //-- draw a corner on the selected cluster
            ((Visualiser) _visualObjs.peek()).drawCorner(r, _offScreenImage,
                this, x, y, _width, _height);
            //-- setup the data model in the table
            _navigator.setData(((Visualiser) _visualObjs.peek()).getDataManager().getWords(),
                r.getClusterObj());
                
            // and show all documents!
            _navigator.showDocuments();            
                
            repaint();
        }
        setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
    
    
    protected void processMouseEvent(MouseEvent e)
    {
        if (e.getID()==MouseEvent.MOUSE_CLICKED)
        {       
            
        if (!_visualObjs.empty())
        {
            Font weak = new Font("Arial",Font.PLAIN,14);
            Font bold = new Font("Arial",Font.BOLD,14);
            
            final ClusterRect r = ((Visualiser) _visualObjs.peek()).inspect(e.getX(),e.getY());
            ClusterObj cObj = r.getClusterObj();
            int numDocs = cObj.getDocsSize();
            
            boolean img_hasChanged;
                    
            popup.removeAll();
            MenuItem menuItem = new MenuItem(numDocs + " documents");
            menuItem.setFont(bold);
            
            menuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent actionEvent)
                {
                       
                    if (r!=null)
                    {
                        
                        //-- draw a corner on the selected cluster
                //        ((Visualiser) _visualObjs.peek()).drawCorner(r, _offScreenImage,
                //            this, x, y, _width, _height);
                        //-- setup the data model in the table
                        
                        String str = r.getClusterObj().getLabel();
                        _displayFrame.log.log("SELECTED_CLUSTER",str);
                        _navigator.setData(((Visualiser) _visualObjs.peek()).getDataManager().getWords(),
                        r.getClusterObj());
                
                        // and show all documents!
                        _navigator.showDocuments();            
                
                        repaint();
                    }
         
                }
            });
            
            popup.add(menuItem);
            popup.addSeparator();
                  
            menuItem = new MenuItem("Refined Query");
            menuItem.setFont(bold);
            menuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent actionEvent)
                {
                    setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                    
                    // all this does is return the rectangle, on which drill down
                    // is to be performed
                    //ClusterRect r = ((Visualiser) _visualObjs.peek()).drillDown(x,y);
                    //-- chose which cluster to drill down
                    if (r!=null)
                    {
                        //-- do sth on 'r'??? send it back to the server and do the clustering again
                        _popUpButton.setEnabled(true);
                        clearDisplay();
                        _drillDownFlag = true;
                        _newTerms = r.getClusterObj().getLabel();
                        _navigator.clear();  //-- clear the table and the display
                        repaint();
                    }
                    else
                    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                }
            });
            
            popup.add(menuItem);
            popup.addSeparator();
            String label = cObj.getLabel();
            
            StringTokenizer st = new StringTokenizer(label);
                       
            while( st.hasMoreTokens() )
            {
                menuItem = new MenuItem(st.nextToken());
                menuItem.setFont(bold);
                menuItem.setEnabled(false);
                popup.add(menuItem);
            }
                        
            popup.show(this,e.getX(),e.getY());
            
            /*img_hasChanged = ((Visualiser) _visualObjs.peek()).drawTipBox(
                _offScreenImage, this, e.getX(),e.getY(), _width,_height);
            System.out.println("Changed?: " + img_hasChanged);
            if (img_hasChanged)*/  repaint();
        }
        
        }  // if mouseclick
    }


    //------------------------------------------------------------------

    public void paint(Graphics oldg)
    {
        //System.out.println("Tree map paint called");
        Graphics2D tmpGph, g = (Graphics2D) oldg;
        Image tmpImg;
        /**
        * create off-scrren images for double-buffering
        * There is a known deficiency bug (Java itself) with createImage in java applications.
        * Workaround: to create necessary image buffers inside paint!
        */
        if (!_init_done){
            _offScreenImage = createImage(_width, _height);
            _init_done = true;
        }

        /*if(this.isVisible())*/ g.drawImage(_offScreenImage,0,0,this);

    }  //-- paint



}  //-- TreemapPanel


        /*
        addMouseMotionListener( new MouseMotionAdapter()
        {
            public void mouseMoved(MouseEvent e)
            {
                if (!_visualObjs.empty())
                {
                    final ClusterRect r = ((Visualiser) _visualObjs.peek()).inspect(e.getX(),e.getY());
                    ClusterObj cObj = r.getClusterObj();
                    int numDocs = cObj.getDocsSize();
                    System.out.print("Mouse moved to ");           
                    System.out.println(e.getX() + " " + e.getY());
                    
                    boolean img_hasChanged;
            
                    popup.removeAll();
                    MenuItem menuItem = new MenuItem(numDocs + " documents");
                    menuItem.addActionListener(new ActionListener()
                    {
                        public void actionPerformed(ActionEvent actionEvent)
                        {
                            System.out.println("Show documents");
                    
                            if (r!=null)
                            {
                                //-- draw a corner on the selected cluster
                                //((Visualiser) _visualObjs.peek()).drawCorner(r, _offScreenImage,
                                // this, x, y, _width, _height);
                                //-- setup the data model in the table
                                _navigator.setData(((Visualiser) _visualObjs.peek()).getDataManager().getWords(),
                                r.getClusterObj());
                
                                // and show all documents!
                                _navigator.showDocuments();            
                
                                 repaint();
                            }
         
                        }
                    });
            
                    popup.add(menuItem);
                
                    menuItem = new MenuItem("Drill down");
                    menuItem.addActionListener(new ActionListener()
                    {
                        public void actionPerformed(ActionEvent actionEvent)
                        {
                            System.out.println("Drill down clicked");
                        }
                    });
            
                    popup.add(menuItem);
            
                    popup.show(this,e.getX(),e.getY());
            
                    img_hasChanged = ((Visualiser) _visualObjs.peek()).drawTipBox(
                    _offScreenImage, this, e.getX(),e.getY(), _width,_height);
                    System.out.println("Changed?: " + img_hasChanged);
                    if (img_hasChanged)  repaint();
                }
            }
        });*/
        
        /*  protected void processMouseEvent(MouseEvent e)
    {

        if (!_visualObjs.empty())
        {
            //-- if the mouse points outside the obj, then copy the original treemap view
            //-- without the tipbox to the screen
            if (e.getID()==MouseEvent.MOUSE_EXITED){
                //-- get the most update view of the treemap
                ((Visualiser) _visualObjs.peek()).drawTreemapView(_offScreenImage, this);
                repaint();
            }
            else if (e.getID()==MouseEvent.MOUSE_CLICKED){

                //-- if the user pressed the first mouse button
                if ((e.getModifiers() & InputEvent.BUTTON1_MASK) == InputEvent.BUTTON1_MASK)
                {
                    System.out.println("Mouse clicked");
                    processInspect(e.getX(),e.getY());
                    
                }
                //-- if the user pressed the third mouse button
                else if ((e.getModifiers() & InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK)
                    processDrillDown(e.getX(),e.getY());

            }
        }

    }*/

    //------------------------------------------------------------------
/*    protected void processMouseMotionEvent(MouseEvent e)
    {
        System.out.println("Outer Mouse in motion");
        if (!_visualObjs.empty())
        {
            Font weak = new Font("Arial",Font.PLAIN,14);
            Font bold = new Font("Arial",Font.BOLD,14);
            
            final ClusterRect r = ((Visualiser) _visualObjs.peek()).inspect(e.getX(),e.getY());
            ClusterObj cObj = r.getClusterObj();
            int numDocs = cObj.getDocsSize();
            
            System.out.println("Inner Mouse in motion");
            System.out.println(e.getX() + " " + e.getY());
            //System.out.println("Mouse in motion");
            boolean img_hasChanged;
                    
            popup.removeAll();
            MenuItem menuItem = new MenuItem(numDocs + " documents");
            menuItem.setFont(bold);
            
            menuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent actionEvent)
                {
                    System.out.println("Show documents");
                    
                    if (r!=null)
                    {
                        //-- draw a corner on the selected cluster
                //        ((Visualiser) _visualObjs.peek()).drawCorner(r, _offScreenImage,
                //            this, x, y, _width, _height);
                        //-- setup the data model in the table
                        _navigator.setData(((Visualiser) _visualObjs.peek()).getDataManager().getWords(),
                        r.getClusterObj());
                
                        // and show all documents!
                        _navigator.showDocuments();            
                
                        repaint();
                    }
         
                }
            });
            
            popup.add(menuItem);
            popup.addSeparator();
                  
            menuItem = new MenuItem("Refined Query");
            menuItem.setFont(bold);
            menuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent actionEvent)
                {
                    setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
                    
                    // all this does is return the rectangle, on which drill down
                    // is to be performed
                    //ClusterRect r = ((Visualiser) _visualObjs.peek()).drillDown(x,y);
                    //-- chose which cluster to drill down
                    if (r!=null)
                    {
                        //-- do sth on 'r'??? send it back to the server and do the clustering again
                        _popUpButton.setEnabled(true);
                        clearDisplay();
                        _drillDownFlag = true;
                        _newTerms = r.getClusterObj().getLabel();
                        _navigator.clear();  //-- clear the table and the display
                        repaint();
                    }
                    else
                    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
                }
            });
            
            popup.add(menuItem);
            popup.addSeparator();
            String label = cObj.getLabel();
            
            StringTokenizer st = new StringTokenizer(label);
                       
            while( st.hasMoreTokens() )
            {
                menuItem = new MenuItem(st.nextToken());
                menuItem.setFont(bold);
                menuItem.setEnabled(false);
                popup.add(menuItem);
            }
                        
            popup.show(this,e.getX(),e.getY());
            
            //img_hasChanged = ((Visualiser) _visualObjs.peek()).drawTipBox(
            //   _offScreenImage, this, e.getX(),e.getY(), _width,_height);
            //System.out.println("Changed?: " + img_hasChanged);
            //if (img_hasChanged)
            repaint();
        }
    }*/
